@@ -1,5 +1,5 @@
 /* challenge.h --- Generate a CRAM-MD5 challenge string.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2008  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with GNU SASL Library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -27,7 +27,8 @@
 
 /* Store zero terminated CRAM-MD5 challenge in output buffer.  The
    CHALLENGE buffer must be allocated by the caller, and must have
-   room for CRAM_MD5_CHALLENGE_LEN characters.  */
-extern void cram_md5_challenge (char challenge[CRAM_MD5_CHALLENGE_LEN]);
+   room for CRAM_MD5_CHALLENGE_LEN characters.  Returns 0 on success,
+   and -1 on randomness problems.  */
+extern int cram_md5_challenge (char challenge[CRAM_MD5_CHALLENGE_LEN]);
 
 #endif /* CHALLENGE_H */
