@@ -1,5 +1,5 @@
 /* listmech.c --- List active client and server mechanisms.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2008  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -13,9 +13,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * License along with GNU SASL Library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License License along with GNU SASL Library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -44,10 +45,7 @@ _gsasl_listmech (Gsasl * ctx,
 
       if (rc == GSASL_OK)
 	{
-	  if (clientp)
-	    gsasl_finish (sctx);
-	  else
-	    gsasl_finish (sctx);
+	  gsasl_finish (sctx);
 
 	  strcat (list, mechs[i].name);
 	  if (i < n_mechs - 1)
@@ -70,7 +68,7 @@ _gsasl_listmech (Gsasl * ctx,
  * allocated by this function, and it is the responsibility of caller
  * to deallocate it.
  *
- * Return value: Returns GSASL_OK if successful, or error code.
+ * Return value: Returns %GSASL_OK if successful, or error code.
  **/
 int
 gsasl_client_mechlist (Gsasl * ctx, char **out)
@@ -89,7 +87,7 @@ gsasl_client_mechlist (Gsasl * ctx, char **out)
  * allocated by this function, and it is the responsibility of caller
  * to deallocate it.
  *
- * Return value: Returns GSASL_OK if successful, or error code.
+ * Return value: Returns %GSASL_OK if successful, or error code.
  **/
 int
 gsasl_server_mechlist (Gsasl * ctx, char **out)
