@@ -13,17 +13,18 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * License along with GNU SASL Library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License License along with GNU SASL Library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
 #include "internal.h"
 
-#if WITH_SASLPREP
+#if HAVE_LIBIDN
 # include <stringprep.h>
-# if HAVE_PR29_H
+# if defined (HAVE_PR29_H) && defined (HAVE_PR29_8Z)
 #  include <pr29.h>
 # endif
 #endif
@@ -47,7 +48,7 @@ int
 gsasl_saslprep (const char *in, Gsasl_saslprep_flags flags,
 		char **out, int *stringpreprc)
 {
-#if WITH_SASLPREP
+#if HAVE_LIBIDN
   int rc;
 
   rc = stringprep_profile (in, out, "SASLprep",
@@ -63,8 +64,8 @@ gsasl_saslprep (const char *in, Gsasl_saslprep_flags flags,
       return GSASL_SASLPREP_ERROR;
     }
 
-# if HAVE_PR29_8Z
-  if (pr29_8z (out) != PR29_SUCCESS)
+# if defined (HAVE_PR29_8Z) && defined (HAVE_PR29_H)
+  if (pr29_8z (*out) != PR29_SUCCESS)
     {
       free (*out);
       *out = NULL;
