@@ -36,20 +36,15 @@
 #include <sys/types.h>
 #include <sys/socket.h>	/* for AF_INET */
 #include <sys/select.h> /* select() */
-
+#include <sys/time.h>	/* select() */
 #include <unistd.h>
+#include <netinet/in.h>
 #ifdef HAVE_NETDB_H
 # include <netdb.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>		/* select() */
-#endif
 #ifdef HAVE_PWD_H
 # include <pwd.h>		/* getpwnam */
 #endif
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
 
 #include <gsasl.h>
 
